/* cmd_global_register.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_global.h"

/*
* argv[0] - channel
* argv[1] - nick / *auth
* argv[2] - (optional) bot nick
*/
static AUTHLOOKUP_CALLBACK(global_cmd_register_auth_lookup);
static USERAUTH_CALLBACK(global_cmd_register_nick_lookup);
static void global_cmd_register_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, char *channel, char *auth, int multibot, char *botname);

struct global_cmd_register_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct ChanNode *chan;
    struct Event *event;
    char *nick;
    char *channel;
    char *botname;
    int multibot;
};

CMD_BIND(global_cmd_register) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *channel = argv[0];
    char *botname = (argc > 2 ? argv[2] : NULL);
    int multibot = 0;
    if(!is_valid_chan(channel)) {
        reply(textclient, user, "NS_INVALID_CHANNEL_NAME", argv[0]);
        return;
    }
    printf_mysql_query("SELECT `botid`, `botclass` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` LEFT JOIN `channels` ON `bot_channels`.`chanid` = `channels`.`channel_id` WHERE `channel_name` = '%s'", escape_string(channel));
    res = mysql_use();
    while ((row = mysql_fetch_row(res)) != NULL) {
        if(atoi(row[1]) == client->botid && (client->botid || client->clientid == atoi(row[0]))) {
            reply(textclient, user, "NS_REGISTER_ALREADY", argv[0], client->user->nick);
            return;
        } else
            multibot = 1;
    }
    printf_mysql_query("SELECT `user_user`, `dnr_timeout`, `dnr_reason`, `dnr_id` FROM `donotregister` LEFT JOIN `users` ON `dnr_user` = `user_id` WHERE `dnr_target` = '%s'", escape_string(channel));
    res = mysql_use();
    if((row = mysql_fetch_row(res)) != NULL) {
        int expire_time = atoi(row[1]);
        if(expire_time) {
            if(expire_time - time(0) <= 0) {
                printf_mysql_query("DELETE FROM `donotregister` WHERE `dnr_id` = '%s'", row[3]);
            } else {
                char expireBuf[MAXLEN];
                reply(textclient, user, "NS_DNR_SET_EXPIRES", channel, row[0], timeToStr(user, (expire_time - time(0)), 2, expireBuf), row[2]);
                return;
            }
        } else {
            reply(textclient, user, "NS_DNR_SET", channel, row[0], row[2]);
            return;
        }
    }
    //if theres already another bot in the channel we don't need a owner parameter...
    if(multibot && argc < 2) {
        //skip all these owner check lines
        multibot = 2;
        global_cmd_register_async1(client, textclient, user, chan, event, channel, NULL, multibot, botname);
        return;
    } else if(argc < 2) {
        global_cmd_register_async1(client, textclient, user, chan, event, channel, user->auth, multibot, botname);
        return;
    }
    //check own access
    if(argv[1][0] == '*') {
        //we've got an auth
        argv[1]++;
        printf_mysql_query("SELECT `user_user` FROM `users` WHERE `user_user` = '%s'", escape_string(argv[1]));
        res = mysql_use();
        if ((row = mysql_fetch_row(res)) != NULL) {
            global_cmd_register_async1(client, textclient, user, chan, event, channel, row[0], multibot, botname);
        } else {
            //we need to create a new user...
            //but first lookup the auth to check if it really exists
            struct global_cmd_register_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                printf_log("global", LOG_ERROR, "%s:%d malloc() failed", __FILE__, __LINE__);
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->chan = chan;
            cache->event = event;
            cache->nick = strdup(argv[1]);
            cache->channel = strdup(channel);
            cache->multibot = multibot;
            cache->botname = (botname ? strdup(botname) : NULL);
            lookup_authname(argv[1], module_id, global_cmd_register_auth_lookup, cache);
        }
    } else {
        struct UserNode *cuser = getUserByNick(argv[1]);
        if(!cuser) {
            cuser = createTempUser(argv[1]);
            if(!cuser) {
                reply(textclient, user, "NS_USER_UNKNOWN", argv[1]);
                return;
            }
            cuser->flags |= USERFLAG_ISTMPUSER;
        }
        if(cuser->flags & USERFLAG_ISAUTHED) {
            global_cmd_register_async1(client, textclient, user, chan, event, channel, cuser->auth, multibot, botname);
        } else {
            struct global_cmd_register_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                printf_log("global", LOG_ERROR, "%s:%d malloc() failed", __FILE__, __LINE__);
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->chan = chan;
            cache->event = event;
            cache->nick = strdup(argv[1]);
            cache->channel = strdup(channel);
            cache->multibot = multibot;
            cache->botname = (botname ? strdup(botname) : NULL);
            get_userauth(cuser, module_id, global_cmd_register_nick_lookup, cache);
        }
    }
}

static AUTHLOOKUP_CALLBACK(global_cmd_register_auth_lookup) {
    struct global_cmd_register_cache *cache = data;
    if(!exists) {
        //AUTH_DOES_NOT_EXIST
        reply(cache->textclient, cache->user, "NS_AUTH_UNKNOWN", cache->nick);
    } else
        global_cmd_register_async1(cache->client, cache->textclient, cache->user, cache->chan, cache->event, cache->channel, auth, cache->multibot, cache->botname);
    if(cache->botname)
        free(cache->botname);
    free(cache->channel);
    free(cache->nick);
    free(cache);
}

static USERAUTH_CALLBACK(global_cmd_register_nick_lookup) {
    struct global_cmd_register_cache *cache = data;
    if(!user) {
        //USER_DOES_NOT_EXIST
        reply(cache->textclient, cache->user, "NS_USER_UNKNOWN", cache->nick);
    }
    else if(!(user->flags & USERFLAG_ISAUTHED)) {
        //USER_NOT_AUTHED
        reply(cache->textclient, cache->user, "NS_USER_NEED_AUTH", cache->nick);
    }
    else
        global_cmd_register_async1(cache->client, cache->textclient, cache->user, cache->chan, cache->event, cache->channel, user->auth, cache->multibot, cache->botname);
    if(cache->botname)
        free(cache->botname);
    free(cache->channel);
    free(cache->nick);
    free(cache);
}

static void global_cmd_register_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, struct Event *event, char *channel, char *auth, int multibot, char *botname) {
    //we've got a valid auth now...
    MYSQL_RES *res;
    MYSQL_ROW row, row2;
    int userid = 0, adminid;
    printf_mysql_query("SELECT `user_id` FROM `users` WHERE `user_user` = '%s'", escape_string(user->auth));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL)
        adminid = atoi(row[0]);
    else
        adminid = 0;
    if(multibot != 2) {
        printf_mysql_query("SELECT `user_user`, `dnr_timeout`, `dnr_reason`, `dnr_id` FROM `donotregister` LEFT JOIN `users` ON `dnr_user` = `user_id` WHERE `dnr_target` = '%s'", escape_string(auth));
        res = mysql_use();
        if((row = mysql_fetch_row(res)) != NULL) {
            int expire_time = atoi(row[1]);
            if(expire_time) {
                if(expire_time - time(0) <= 0) {
                    printf_mysql_query("DELETE FROM `donotregister` WHERE `dnr_id` = '%s'", row[3]);
                } else {
                    char expireBuf[MAXLEN];
                    reply(textclient, user, "NS_DNR_SET_EXPIRES", auth, row[0], timeToStr(user, (expire_time - time(0)), 2, expireBuf), row[2]);
                    return;
                }
            } else {
                reply(textclient, user, "NS_DNR_SET", auth, row[0], row[2]);
                return;
            }
        }
        printf_mysql_query("SELECT `user_id` FROM `users` WHERE `user_user` = '%s'", escape_string(auth));
        res = mysql_use();
        if ((row = mysql_fetch_row(res)) != NULL) {
            userid = atoi(row[0]);
        } else {
            printf_mysql_query("INSERT INTO `users` (`user_user`) VALUES ('%s')", escape_string(auth));
            userid = (int) mysql_insert_id(get_mysql_conn());
        }
    }
    if(client->botid)
        printf_mysql_query("SELECT `id`, `max_channels`, `defaulttrigger`, `nick` FROM `bots` WHERE `botclass` = '%d' AND `active` = '1' ORDER BY `register_priority` DESC", client->botid);
    else
        printf_mysql_query("SELECT `id`, `max_channels`, `defaulttrigger`, `nick` FROM `bots` WHERE `id` = '%d' AND `active` = '1'", client->clientid);
    res = mysql_use();
    int botid = 0;
    while ((row = mysql_fetch_row(res)) != NULL) {
        //check channel count
        printf_mysql_query("SELECT COUNT(*) FROM `bot_channels` WHERE `botid` = '%s'", row[0]);
        row2 = mysql_fetch_row(mysql_use());
        if(atoi(row2[0]) < atoi(row[1]) && (!botname || !stricmp(botname, row[3]))) {
            botid = atoi(row[0]);
            break;
        }
    }
    if(!botid) {
        reply(textclient, user, "NS_REGISTER_FULL");
        return;
    }
    int chanid;
    printf_mysql_query("SELECT `channel_id` FROM `channels` WHERE `channel_name` = '%s'", escape_string(channel));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        chanid = atoi(row[0]);
        printf_mysql_query("UPDATE `channels` SET `channel_registered` = UNIX_TIMESTAMP(), `channel_registrator` = '%d' WHERE `channel_id` = '%d'", adminid, chanid);
    } else {
        printf_mysql_query("INSERT INTO `channels` (`channel_name`, `channel_registered`, `channel_registrator`) VALUES ('%s', UNIX_TIMESTAMP(), '%d')", escape_string(channel), adminid);
        chanid = (int) mysql_insert_id(get_mysql_conn());
    }
    struct ClientSocket *bot;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(bot->clientid == botid)
            break;
    }
    if(bot) {
        putsock(bot, "JOIN %s", channel);
    } else
        reply(textclient, user, "NS_REGISTER_DISCONNECTED", channel);
    printf_mysql_query("INSERT INTO `bot_channels` (`botid`, `chanid`, `trigger`) VALUES ('%d', '%d', NULL)", botid, chanid);
    if(multibot != 2) {
        if(multibot) {
            printf_mysql_query("UPDATE `chanusers` SET `chanuser_access` = 499 WHERE `chanuser_cid` = '%d' AND `chanuser_access` = '500'", chanid);
            printf_mysql_query("DELETE FROM `chanusers` WHERE `chanuser_cid` = '%d' AND `chanuser_uid` = '%d'", chanid, userid);
        } else
            printf_mysql_query("DELETE FROM `chanusers` WHERE `chanuser_cid` = '%d'", chanid);
        printf_mysql_query("INSERT INTO `chanusers` (`chanuser_cid`, `chanuser_uid`, `chanuser_access`) VALUES ('%d', '%d', '%d')", chanid, userid, 500);
        reply(textclient, user, "NS_REGISTER_DONE", channel, auth);
    } else
        reply(textclient, user, "NS_REGISTER_DONE_NOAUTH", channel);
    // NeonBackup SubBlock
    if(client->botid == NEONSERV_BOTID) {
        char setting[128];
        sprintf(setting, "modules.%s.auto_backup_register", get_module_name(module_id));
        if(get_int_field(setting))
            module_global_cmd_register_neonbackup(channel);
    }
    logEvent(event);
}

void global_cmd_register_neonbackup(char *channel) {
    MYSQL_RES *res;
    MYSQL_ROW row, row2;
    printf_mysql_query("SELECT `botid` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` LEFT JOIN `channels` ON `bot_channels`.`chanid` = `channels`.`channel_id` WHERE `channel_name` = '%s' AND botclass = '%d'", escape_string(channel), NEONBACKUP_BOTID);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL)
        return;
    int chanid;
    printf_mysql_query("SELECT `channel_id` FROM `channels` WHERE `channel_name` = '%s'", escape_string(channel));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        chanid = atoi(row[0]);
    } else
        return;
    printf_mysql_query("SELECT `id`, `max_channels`, `defaulttrigger`, `nick` FROM `bots` WHERE `botclass` = '%d' AND `active` = '1' ORDER BY `register_priority` DESC", NEONBACKUP_BOTID);
    res = mysql_use();
    int botid = 0;
    while ((row = mysql_fetch_row(res)) != NULL) {
        //check channel count
        printf_mysql_query("SELECT COUNT(*) FROM `bot_channels` WHERE `botid` = '%s'", row[0]);
        row2 = mysql_fetch_row(mysql_use());
        if(atoi(row2[0]) < atoi(row[1])) {
            botid = atoi(row[0]);
            break;
        }
    }
    if(!botid)
        return;
    struct ClientSocket *bot;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(bot->clientid == botid)
            break;
    }
    if(bot) {
        putsock(bot, "JOIN %s", channel);
    }
    printf_mysql_query("INSERT INTO `bot_channels` (`botid`, `chanid`, `trigger`) VALUES ('%d', '%d', NULL)", botid, chanid);
}

