/* cmd_global_unregister.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_global.h"
#include "../botid.h"

/*
* argv[0] - channel
*/
CMD_BIND(global_cmd_unregister) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *channel;
    if(argc)
        channel = argv[0];
    else
        channel = (chan ? chan->name : "");
    if(!is_valid_chan(channel)) {
        reply(textclient, user, "NS_INVALID_CHANNEL_NAME", channel);
        return;
    }
    int chanid;
    printf_mysql_query("SELECT `channel_id`, `channel_nodelete` FROM `channels` WHERE `channel_name` = '%s'", escape_string(channel));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        chanid = atoi(row[0]);
    } else {
        reply(textclient, user, "NS_UNREGISTER_NOT_REGISTERED", channel, client->user->nick);
        return;
    }
    if(client->botid == NEONSERV_BOTID && !strcmp(row[1], "1")) {
        reply(textclient, user, "NS_UNREGISTER_NODELETE", channel);
        return;
    }
    int sync_neonspam_unreg = get_int_field("General.sync_neonspam_unreg");
    if(client->botid == 0)
        printf_mysql_query("SELECT `botid`, `bot_channels`.`id`, `suspended` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` WHERE `chanid` = '%d' AND `botclass` = '0' AND `botid` = '%d'", chanid, client->clientid);
    else
        printf_mysql_query("SELECT `botid`, `bot_channels`.`id`, `suspended` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` WHERE `chanid` = '%d' AND `botclass` = '%d'", chanid, client->botid);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) == NULL) {
        reply(textclient, user, "NS_UNREGISTER_NOT_REGISTERED", channel, client->user->nick);
        return;
    }
    int botid = atoi(row[0]);
    struct ClientSocket *bot;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(bot->clientid == botid)
            break;
    }
    printf_mysql_query("DELETE FROM `bot_channels` WHERE `id` = '%s'", row[1]);
    reply(textclient, user, "NS_UNREGISTER_DONE", channel);
    if(bot && strcmp(row[2], "1")) {
        putsock(bot, "PART %s :Channel unregistered.", channel);
    }
    if(client->botid == NEONSERV_BOTID && sync_neonspam_unreg) {
        printf_mysql_query("SELECT `botid`, `bot_channels`.`id`, `suspended` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` LEFT JOIN `channels` ON `chanid` = `channel_id` WHERE `channel_name` = '%s' AND `botclass` = '%d'", escape_string(channel), NEONSPAM_BOTID);
        res = mysql_use();
        if ((row = mysql_fetch_row(res))) {
            botid = atoi(row[0]);
            for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
                if(bot->clientid == botid)
                    break;
            }
            printf_mysql_query("DELETE FROM `bot_channels` WHERE `id` = '%s'", row[1]);
            if(bot && strcmp(row[2], "1"))
                putsock(bot, "PART %s :Channel unregistered.", channel);
        }
    }
    // NeonBackup SubBlock
    if(client->botid == NEONSERV_BOTID) {
        char setting[128];
        sprintf(setting, "modules.%s.auto_backup_unregister", get_module_name(module_id));
        if(get_int_field(setting))
            module_global_cmd_unregister_neonbackup(channel);
    }
    logEvent(event);
}

void global_cmd_unregister_neonbackup(char *channel) {
	MYSQL_RES *res;
    MYSQL_ROW row;
    int chanid;
    printf_mysql_query("SELECT `channel_id` FROM `channels` WHERE `channel_name` = '%s'", escape_string(channel));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        chanid = atoi(row[0]);
    } else 
        return;
    printf_mysql_query("SELECT `botid`, `bot_channels`.`id`, `suspended` FROM `bot_channels` LEFT JOIN `bots` ON `bot_channels`.`botid` = `bots`.`id` WHERE `chanid` = '%d' AND `botclass` = '%d'", chanid, NEONBACKUP_BOTID);
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) == NULL)
        return;
    int botid = atoi(row[0]);
    struct ClientSocket *bot;
    for(bot = getBots(SOCKET_FLAG_READY, NULL); bot; bot = getBots(SOCKET_FLAG_READY, bot)) {
        if(bot->clientid == botid)
            break;
    }
    printf_mysql_query("DELETE FROM `bot_channels` WHERE `id` = '%s'", row[1]);
    if(bot && strcmp(row[2], "1")) {
        putsock(bot, "PART %s :Channel unregistered.", channel);
    }
}
